==> mule-artifact-cfg-1.2.23-20250920.201356-11-sources.jar: org/mule/runtime/cfg/internal/node/ChainedExecutionPathNodeBuilder.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.cfg.internal.node;

import static java.util.Optional.empty;
import static java.util.Optional.of;

import org.mule.runtime.ast.api.ComponentAst;
import org.mule.runtime.cfg.api.ChainExecutionPathTree;
import org.mule.runtime.cfg.api.ChainExecutionPathTreeVisitor;

import java.util.Optional;
import java.util.function.Predicate;


public class ChainedExecutionPathNodeBuilder {

  private final ComponentAst chainAst;
  private ChainNode first;
  private ChainNode last;
  private boolean continueAdding = true;

  public ChainedExecutionPathNodeBuilder(ComponentAst chainAst) {
    this.chainAst = chainAst;
  }

  public ChainedExecutionPathNodeBuilder addChild(ChainExecutionPathTree child) {
    if (!continueAdding) {
      return this;
    }
    ChainNode newNode = new ChainNode(child);
    if (first == null) {
      first = newNode;
    } else {
      last.next = of(newNode);
    }
    last = newNode;

    // WE ARE NOT CONSIDERING IF THE ERROR IS HANDLED WITHIN THIS CONTEXT! (TODO W-12390971)
    continueAdding =
        !child.allExecutionPathsContain(ast -> ast.getComponentAst().getIdentifier().getName().equals("raise-error"));
    return this;
  }

  public ChainExecutionPathTree build() {
    return new HeadNode(chainAst, first != null ? first : new ChainNode(new NullNode()));
  }

  public static class HeadNode implements ChainExecutionPathTree {

    private final ComponentAst chainAst;
    private final ChainNode firstNode;

    public HeadNode(ComponentAst chainAst, ChainNode firstNode) {
      this.chainAst = chainAst;
      this.firstNode = firstNode;
    }

    @Override
    public void accept(ChainExecutionPathTreeVisitor visitor) {
      this.firstNode.accept(visitor);
    }

    @Override
    public ComponentAst getComponentAst() {
      return this.chainAst;
    }

    @Override
    public boolean anyExecutionPathContains(Predicate<ChainExecutionPathTree> predicate) {
      return firstNode.anyExecutionPathContains(predicate);
    }

    @Override
    public boolean allExecutionPathsContain(Predicate<ChainExecutionPathTree> predicate) {
      return firstNode.allExecutionPathsContain(predicate);
    }
  }

  private static class ChainNode implements ChainExecutionPathTree {

    private final ChainExecutionPathTree delegate;
    public Optional<ChainNode> next = empty();

    private ChainNode(ChainExecutionPathTree delegate) {
      this.delegate = delegate;
    }

    @Override
    public void accept(ChainExecutionPathTreeVisitor visitor) {
      this.delegate.accept(visitor);
      this.next.ifPresent(node -> node.accept(visitor));
    }

    @Override
    public ComponentAst getComponentAst() {
      return this.delegate.getComponentAst();
    }

    @Override
    public boolean anyExecutionPathContains(Predicate<ChainExecutionPathTree> predicate) {
      return this.delegate.anyExecutionPathContains(predicate)
          || this.next.map(node -> node.anyExecutionPathContains(predicate)).orElse(false);
    }

    @Override
    public boolean allExecutionPathsContain(Predicate<ChainExecutionPathTree> predicate) {
      return this.delegate.allExecutionPathsContain(predicate)
          || this.next.map(node -> node.allExecutionPathsContain(predicate)).orElse(false);
    }

  }

}

==> mule-artifact-cfg-1.4.10-20251105.141616-7-sources.jar: org/mule/runtime/cfg/internal/node/ChainedExecutionPathNodeBuilder.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.cfg.internal.node;

import static java.util.Collections.unmodifiableList;
import static java.util.Optional.empty;
import static java.util.Optional.of;

import org.mule.runtime.ast.api.ComponentAst;
import org.mule.runtime.cfg.api.ChainExecutionPathTree;
import org.mule.runtime.cfg.api.ChainExecutionPathTreeVisitor;
import org.mule.runtime.cfg.internal.node.errorhandling.ErrorHandlerWrapperNode;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.function.Predicate;


public class ChainedExecutionPathNodeBuilder {

  private final ComponentAst chainAst;
  private ChainNode first;
  private ChainNode last;
  private boolean continueAdding = true;
  private final List<ChainExecutionPathTree> children = new ArrayList<>();
  private Optional<ErrorHandlerWrapperNode> ownedErrorHandler = empty();

  public ChainedExecutionPathNodeBuilder(ComponentAst chainAst) {
    this.chainAst = chainAst;
  }

  public ChainedExecutionPathNodeBuilder addChild(ChainExecutionPathTree child) {
    if (!continueAdding) {
      return this;
    }
    children.add(child);
    ChainNode newNode = new ChainNode(child);
    if (first == null) {
      first = newNode;
    } else {
      last.next = of(newNode);
    }
    last = newNode;

    // TODO (W-12390971) Consider if there is a raise-error in all the executions, then we can either not add the
    // next elements, or avoid the executions.
    continueAdding = true;
    return this;
  }

  public ChainedExecutionPathNodeBuilder addOwnedErrorHandler(ErrorHandlerWrapperNode errorHandler) {
    this.ownedErrorHandler = of(errorHandler);
    return this;
  }

  public ChainExecutionPathTree build() {
    return new HeadNode(chainAst, first != null ? first : new ChainNode(NullNode.getInstance()), children, ownedErrorHandler);
  }

  private static class HeadNode implements ChainExecutionPathTree {

    private final ComponentAst chainAst;
    private final ChainNode firstNode;
    private final List<ChainExecutionPathTree> children;
    private final Optional<ErrorHandlerWrapperNode> errorHandlingAfterChain;

    public HeadNode(ComponentAst chainAst, ChainNode firstNode, List<ChainExecutionPathTree> children,
                    Optional<ErrorHandlerWrapperNode> errorHandlingAfterChain) {
      this.chainAst = chainAst;
      this.firstNode = firstNode;
      this.children = children;
      this.errorHandlingAfterChain = errorHandlingAfterChain;
    }

    @Override
    public void accept(ChainExecutionPathTreeVisitor visitor) {
      this.firstNode.accept(visitor);
      this.errorHandlingAfterChain.ifPresent(errorHandler -> errorHandler.accept(visitor));
    }

    @Override
    public ComponentAst getComponentAst() {
      return this.chainAst;
    }

    @Override
    public boolean anyExecutionPathContains(Predicate<ChainExecutionPathTree> predicate) {
      return firstNode.anyExecutionPathContains(predicate);
    }

    @Override
    public boolean allExecutionPathsContain(Predicate<ChainExecutionPathTree> predicate) {
      return firstNode.allExecutionPathsContain(predicate);
    }

    @Override
    public List<ChainExecutionPathTree> children() {
      return unmodifiableList(this.children);
    }
  }

  private static class ChainNode implements ChainExecutionPathTree {

    private final ChainExecutionPathTree delegate;
    public Optional<ChainNode> next = empty();

    private ChainNode(ChainExecutionPathTree delegate) {
      this.delegate = delegate;
    }

    @Override
    public void accept(ChainExecutionPathTreeVisitor visitor) {
      this.delegate.accept(visitor);
      this.next.ifPresent(node -> node.accept(visitor));
    }

    @Override
    public ComponentAst getComponentAst() {
      return this.delegate.getComponentAst();
    }

    @Override
    public boolean anyExecutionPathContains(Predicate<ChainExecutionPathTree> predicate) {
      return this.delegate.anyExecutionPathContains(predicate)
          || this.next.map(node -> node.anyExecutionPathContains(predicate)).orElse(false);
    }

    @Override
    public boolean allExecutionPathsContain(Predicate<ChainExecutionPathTree> predicate) {
      return this.delegate.allExecutionPathsContain(predicate)
          || this.next.map(node -> node.allExecutionPathsContain(predicate)).orElse(false);
    }

  }

}
